/*  ksim - a system monitor for kde
 *
 *  Copyright (C) 2001  Robbie Ward <linuxphreak@gmx.co.uk>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "filesystemstats.h"

#include <qglobal.h>
#include <qfile.h>
#include <kdebug.h>

#include <config.h>

#include <sys/types.h>
#include <sys/param.h>

#ifdef HAVE_SYS_STATVFS_H
#include <sys/statvfs.h>
#elif defined(HAVE_SYS_STATFS_H)
#include <sys/statfs.h>
#endif
#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif
#ifdef HAVE_SYS_MOUNT_H
#include <sys/mount.h>
#endif

#include <stdio.h>
#include <unistd.h>

#ifdef HAVE_STATVFS
#define FSystemStats struct statvfs
#elif defined( HAVE_STATFS ) || defined( __FreeBSD__ )
#define FSystemStats struct statfs
#else
struct ksim_statfs // fall back for (possibly) non-supported systems
{
  int f_blocks;
  int f_bfree;
};

#define FSystemStats struct ksim_statfs
#endif

int fsystemStats( const char * file, FSystemStats & stats )
{
#ifdef HAVE_STATVFS
  return statvfs( file, &stats );
#elif defined( HAVE_STATFS ) || defined( __FreeBSD__ )
  return statfs( file, &stats );
#else // fall back for (possibly) non-supported systems
  (void)file;

  stats.f_blocks = 0;
  stats.f_bfree = 0;
  return -1;
#endif
}

bool FilesystemStats::readStats( const QString & mntPoint, int & totalBlocks, int & freeBlocks )
{
  FSystemStats sysStats;
  if ( fsystemStats( QFile::encodeName( mntPoint ).data(), sysStats ) < 0 )
  {
    kdError() << "While reading filesystem information for " << mntPoint << endl;
    totalBlocks = 0;
    freeBlocks = 0;
  }

  totalBlocks = sysStats.f_blocks;
  freeBlocks = sysStats.f_bfree;

  // Return true if our filesystem is statable
  return totalBlocks > 0;
}
